package com.googlecode.functionalcollections;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Set;

import javax.annotation.Nullable;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterables;


/**
 * Functional extension of {@link Iterable}, most methods are simple wrapper of {@link Iterables}.
 * 
 * @author James Wang
 * @see Iterable
 * @see Iterables
 * 
 */
public interface FunctionalIterable<F> extends Iterable<F> {
	
	FunctionalIterable<F> each(Block<? super F> block);

	boolean any(Predicate<? super F> predicate);
	
	boolean all(Predicate<? super F> predicate);
	
	/**
	 * @see Iterables#find(Iterable, Predicate)
	 * @see #detect(Predicate)
	 */
	F find(Predicate<? super F> predicate) throws NoSuchElementException;

	/**
	 * Returns the first element in that satisfies the given predicate, <code>null</code>
	 * if no element satisfies.
	 * @see #find(Predicate)
	 */
	F detect(Predicate<? super F> predicate);
	
	/**
	 * 
	 * Alias method of {@link #filter(Predicate)}.
	 * 
	 */
	FunctionalIterable<F> select(Predicate<? super F> predicate);
	
	FunctionalIterable<F> filter(Predicate<? super F> predicate);

	/**
	 * Alias method of {@link #transform(Function)}.
	 */
	<T> FunctionalIterable<T> map(Function<? super F, ? extends T> function);
	
	<T> FunctionalIterable<T> transform(Function<? super F, ? extends T> function);
	
	<M> M inject(M initial, Injector<M, F> injector);
	
	int size();
	
	boolean contains(Object element);
	
	boolean removeAll(Collection<?> elementsToRemove);
	
	boolean retainAll(Collection<?> elementsToRetain);
	
	boolean elementsEqual(Iterable<?> target);
	
	boolean elementsEqual(Iterable<?> target, EqualDelegate equalDelegate);
	
	String toString();
	
	F getOnlyElement();
	
	F getOnlyElement(@Nullable F defaultValue);
	
	F[] toArray(Class<F> type);
	
	Collection<F> toCollection();
	
	List<F> toList();
	
	Set<F> toSet();
	
	int frequency(Object element);
	
	FunctionalIterable<F> uniq();
	
	FunctionalIterable<F> cycle();
	
	FunctionalIterable<F> reverse();
	
	FunctionalIterable<F> concat(Iterable<? extends F> target);
	
	FunctionalIterable<F> concat(F... target);
	
	FunctionalIterable<List<F>> partition(int size);
	
	FunctionalIterable<List<F>> paddedPartition(int size);
	
	F get(int position);
	
	F getLast();
	
	boolean isEmpty();

	Iterator<F> messagedIterator(String messageOnLast);
	
	/**
	 * @see Joiner
	 * @see #join(JoinSpecification)
	 */
	String join(String separator);
	
	/**
	 * @see Joiner
	 */
	String join(Joiner joiner);


//	FunctionalIterable<F> flatten();

}
